from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RotationalAccelerationUnits(Enum):
        """
            RotationalAccelerationUnits enumeration
        """
        
        RadianPerSecondSquared = 'RadianPerSecondSquared'
        """
            
        """
        
        DegreePerSecondSquared = 'DegreePerSecondSquared'
        """
            
        """
        
        RevolutionPerMinutePerSecond = 'RevolutionPerMinutePerSecond'
        """
            
        """
        
        RevolutionPerSecondSquared = 'RevolutionPerSecondSquared'
        """
            
        """
        

class RotationalAccelerationDto:
    """
    A DTO representation of a RotationalAcceleration

    Attributes:
        value (float): The value of the RotationalAcceleration.
        unit (RotationalAccelerationUnits): The specific unit that the RotationalAcceleration value is representing.
    """

    def __init__(self, value: float, unit: RotationalAccelerationUnits):
        """
        Create a new DTO representation of a RotationalAcceleration

        Parameters:
            value (float): The value of the RotationalAcceleration.
            unit (RotationalAccelerationUnits): The specific unit that the RotationalAcceleration value is representing.
        """
        self.value: float = value
        """
        The value of the RotationalAcceleration
        """
        self.unit: RotationalAccelerationUnits = unit
        """
        The specific unit that the RotationalAcceleration value is representing
        """

    def to_json(self):
        """
        Get a RotationalAcceleration DTO JSON object representing the current unit.

        :return: JSON object represents RotationalAcceleration DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "RadianPerSecondSquared"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RotationalAcceleration DTO from a json representation.

        :param data: The RotationalAcceleration DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "RadianPerSecondSquared"}
        :return: A new instance of RotationalAccelerationDto.
        :rtype: RotationalAccelerationDto
        """
        return RotationalAccelerationDto(value=data["value"], unit=RotationalAccelerationUnits(data["unit"]))


class RotationalAcceleration(AbstractMeasure):
    """
    Angular acceleration is the rate of change of rotational speed.

    Args:
        value (float): The value.
        from_unit (RotationalAccelerationUnits): The RotationalAcceleration unit to create from, The default unit is RadianPerSecondSquared
    """
    def __init__(self, value: float, from_unit: RotationalAccelerationUnits = RotationalAccelerationUnits.RadianPerSecondSquared):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__radians_per_second_squared = None
        
        self.__degrees_per_second_squared = None
        
        self.__revolutions_per_minute_per_second = None
        
        self.__revolutions_per_second_squared = None
        

    def convert(self, unit: RotationalAccelerationUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RotationalAccelerationUnits = RotationalAccelerationUnits.RadianPerSecondSquared) -> RotationalAccelerationDto:
        """
        Get a new instance of RotationalAcceleration DTO representing the current unit.

        :param hold_in_unit: The specific RotationalAcceleration unit to store the RotationalAcceleration value in the DTO representation.
        :type hold_in_unit: RotationalAccelerationUnits
        :return: A new instance of RotationalAccelerationDto.
        :rtype: RotationalAccelerationDto
        """
        return RotationalAccelerationDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RotationalAccelerationUnits = RotationalAccelerationUnits.RadianPerSecondSquared):
        """
        Get a RotationalAcceleration DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RotationalAcceleration unit to store the RotationalAcceleration value in the DTO representation.
        :type hold_in_unit: RotationalAccelerationUnits
        :return: JSON object represents RotationalAcceleration DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "RadianPerSecondSquared"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(rotational_acceleration_dto: RotationalAccelerationDto):
        """
        Obtain a new instance of RotationalAcceleration from a DTO unit object.

        :param rotational_acceleration_dto: The RotationalAcceleration DTO representation.
        :type rotational_acceleration_dto: RotationalAccelerationDto
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration(rotational_acceleration_dto.value, rotational_acceleration_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RotationalAcceleration from a DTO unit json representation.

        :param data: The RotationalAcceleration DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "RadianPerSecondSquared"}
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration.from_dto(RotationalAccelerationDto.from_json(data))

    def __convert_from_base(self, from_unit: RotationalAccelerationUnits) -> float:
        value = self._value
        
        if from_unit == RotationalAccelerationUnits.RadianPerSecondSquared:
            return (value)
        
        if from_unit == RotationalAccelerationUnits.DegreePerSecondSquared:
            return ((180 / math.pi) * value)
        
        if from_unit == RotationalAccelerationUnits.RevolutionPerMinutePerSecond:
            return ((60 / (2 * math.pi)) * value)
        
        if from_unit == RotationalAccelerationUnits.RevolutionPerSecondSquared:
            return ((1 / (2 * math.pi)) * value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RotationalAccelerationUnits) -> float:
        
        if to_unit == RotationalAccelerationUnits.RadianPerSecondSquared:
            return (value)
        
        if to_unit == RotationalAccelerationUnits.DegreePerSecondSquared:
            return ((math.pi / 180) * value)
        
        if to_unit == RotationalAccelerationUnits.RevolutionPerMinutePerSecond:
            return (((2 * math.pi) / 60) * value)
        
        if to_unit == RotationalAccelerationUnits.RevolutionPerSecondSquared:
            return ((2 * math.pi) * value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_radians_per_second_squared(radians_per_second_squared: float):
        """
        Create a new instance of RotationalAcceleration from a value in radians_per_second_squared.

        

        :param meters: The RotationalAcceleration value in radians_per_second_squared.
        :type radians_per_second_squared: float
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration(radians_per_second_squared, RotationalAccelerationUnits.RadianPerSecondSquared)

    
    @staticmethod
    def from_degrees_per_second_squared(degrees_per_second_squared: float):
        """
        Create a new instance of RotationalAcceleration from a value in degrees_per_second_squared.

        

        :param meters: The RotationalAcceleration value in degrees_per_second_squared.
        :type degrees_per_second_squared: float
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration(degrees_per_second_squared, RotationalAccelerationUnits.DegreePerSecondSquared)

    
    @staticmethod
    def from_revolutions_per_minute_per_second(revolutions_per_minute_per_second: float):
        """
        Create a new instance of RotationalAcceleration from a value in revolutions_per_minute_per_second.

        

        :param meters: The RotationalAcceleration value in revolutions_per_minute_per_second.
        :type revolutions_per_minute_per_second: float
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration(revolutions_per_minute_per_second, RotationalAccelerationUnits.RevolutionPerMinutePerSecond)

    
    @staticmethod
    def from_revolutions_per_second_squared(revolutions_per_second_squared: float):
        """
        Create a new instance of RotationalAcceleration from a value in revolutions_per_second_squared.

        

        :param meters: The RotationalAcceleration value in revolutions_per_second_squared.
        :type revolutions_per_second_squared: float
        :return: A new instance of RotationalAcceleration.
        :rtype: RotationalAcceleration
        """
        return RotationalAcceleration(revolutions_per_second_squared, RotationalAccelerationUnits.RevolutionPerSecondSquared)

    
    @property
    def radians_per_second_squared(self) -> float:
        """
        
        """
        if self.__radians_per_second_squared != None:
            return self.__radians_per_second_squared
        self.__radians_per_second_squared = self.__convert_from_base(RotationalAccelerationUnits.RadianPerSecondSquared)
        return self.__radians_per_second_squared

    
    @property
    def degrees_per_second_squared(self) -> float:
        """
        
        """
        if self.__degrees_per_second_squared != None:
            return self.__degrees_per_second_squared
        self.__degrees_per_second_squared = self.__convert_from_base(RotationalAccelerationUnits.DegreePerSecondSquared)
        return self.__degrees_per_second_squared

    
    @property
    def revolutions_per_minute_per_second(self) -> float:
        """
        
        """
        if self.__revolutions_per_minute_per_second != None:
            return self.__revolutions_per_minute_per_second
        self.__revolutions_per_minute_per_second = self.__convert_from_base(RotationalAccelerationUnits.RevolutionPerMinutePerSecond)
        return self.__revolutions_per_minute_per_second

    
    @property
    def revolutions_per_second_squared(self) -> float:
        """
        
        """
        if self.__revolutions_per_second_squared != None:
            return self.__revolutions_per_second_squared
        self.__revolutions_per_second_squared = self.__convert_from_base(RotationalAccelerationUnits.RevolutionPerSecondSquared)
        return self.__revolutions_per_second_squared

    
    def to_string(self, unit: RotationalAccelerationUnits = RotationalAccelerationUnits.RadianPerSecondSquared, fractional_digits: int = None) -> str:
        """
        Format the RotationalAcceleration to a string.
        
        Note: the default format for RotationalAcceleration is RadianPerSecondSquared.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RotationalAcceleration. Default is 'RadianPerSecondSquared'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RotationalAccelerationUnits.RadianPerSecondSquared:
            return f"""{super()._truncate_fraction_digits(self.radians_per_second_squared, fractional_digits)} rad/s²"""
        
        if unit == RotationalAccelerationUnits.DegreePerSecondSquared:
            return f"""{super()._truncate_fraction_digits(self.degrees_per_second_squared, fractional_digits)} °/s²"""
        
        if unit == RotationalAccelerationUnits.RevolutionPerMinutePerSecond:
            return f"""{super()._truncate_fraction_digits(self.revolutions_per_minute_per_second, fractional_digits)} rpm/s"""
        
        if unit == RotationalAccelerationUnits.RevolutionPerSecondSquared:
            return f"""{super()._truncate_fraction_digits(self.revolutions_per_second_squared, fractional_digits)} r/s²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RotationalAccelerationUnits = RotationalAccelerationUnits.RadianPerSecondSquared) -> str:
        """
        Get RotationalAcceleration unit abbreviation.
        Note! the default abbreviation for RotationalAcceleration is RadianPerSecondSquared.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RotationalAccelerationUnits.RadianPerSecondSquared:
            return """rad/s²"""
        
        if unit_abbreviation == RotationalAccelerationUnits.DegreePerSecondSquared:
            return """°/s²"""
        
        if unit_abbreviation == RotationalAccelerationUnits.RevolutionPerMinutePerSecond:
            return """rpm/s"""
        
        if unit_abbreviation == RotationalAccelerationUnits.RevolutionPerSecondSquared:
            return """r/s²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ReciprocalLengthUnits(Enum):
        """
            ReciprocalLengthUnits enumeration
        """
        
        InverseMeter = 'InverseMeter'
        """
            
        """
        
        InverseCentimeter = 'InverseCentimeter'
        """
            
        """
        
        InverseMillimeter = 'InverseMillimeter'
        """
            
        """
        
        InverseMile = 'InverseMile'
        """
            
        """
        
        InverseYard = 'InverseYard'
        """
            
        """
        
        InverseFoot = 'InverseFoot'
        """
            
        """
        
        InverseUsSurveyFoot = 'InverseUsSurveyFoot'
        """
            
        """
        
        InverseInch = 'InverseInch'
        """
            
        """
        
        InverseMil = 'InverseMil'
        """
            
        """
        
        InverseMicroinch = 'InverseMicroinch'
        """
            
        """
        

class ReciprocalLengthDto:
    """
    A DTO representation of a ReciprocalLength

    Attributes:
        value (float): The value of the ReciprocalLength.
        unit (ReciprocalLengthUnits): The specific unit that the ReciprocalLength value is representing.
    """

    def __init__(self, value: float, unit: ReciprocalLengthUnits):
        """
        Create a new DTO representation of a ReciprocalLength

        Parameters:
            value (float): The value of the ReciprocalLength.
            unit (ReciprocalLengthUnits): The specific unit that the ReciprocalLength value is representing.
        """
        self.value: float = value
        """
        The value of the ReciprocalLength
        """
        self.unit: ReciprocalLengthUnits = unit
        """
        The specific unit that the ReciprocalLength value is representing
        """

    def to_json(self):
        """
        Get a ReciprocalLength DTO JSON object representing the current unit.

        :return: JSON object represents ReciprocalLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InverseMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ReciprocalLength DTO from a json representation.

        :param data: The ReciprocalLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InverseMeter"}
        :return: A new instance of ReciprocalLengthDto.
        :rtype: ReciprocalLengthDto
        """
        return ReciprocalLengthDto(value=data["value"], unit=ReciprocalLengthUnits(data["unit"]))


class ReciprocalLength(AbstractMeasure):
    """
    Reciprocal (Inverse) Length is used in various fields of science and mathematics. It is defined as the inverse value of a length unit.

    Args:
        value (float): The value.
        from_unit (ReciprocalLengthUnits): The ReciprocalLength unit to create from, The default unit is InverseMeter
    """
    def __init__(self, value: float, from_unit: ReciprocalLengthUnits = ReciprocalLengthUnits.InverseMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__inverse_meters = None
        
        self.__inverse_centimeters = None
        
        self.__inverse_millimeters = None
        
        self.__inverse_miles = None
        
        self.__inverse_yards = None
        
        self.__inverse_feet = None
        
        self.__inverse_us_survey_feet = None
        
        self.__inverse_inches = None
        
        self.__inverse_mils = None
        
        self.__inverse_microinches = None
        

    def convert(self, unit: ReciprocalLengthUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ReciprocalLengthUnits = ReciprocalLengthUnits.InverseMeter) -> ReciprocalLengthDto:
        """
        Get a new instance of ReciprocalLength DTO representing the current unit.

        :param hold_in_unit: The specific ReciprocalLength unit to store the ReciprocalLength value in the DTO representation.
        :type hold_in_unit: ReciprocalLengthUnits
        :return: A new instance of ReciprocalLengthDto.
        :rtype: ReciprocalLengthDto
        """
        return ReciprocalLengthDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ReciprocalLengthUnits = ReciprocalLengthUnits.InverseMeter):
        """
        Get a ReciprocalLength DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ReciprocalLength unit to store the ReciprocalLength value in the DTO representation.
        :type hold_in_unit: ReciprocalLengthUnits
        :return: JSON object represents ReciprocalLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "InverseMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(reciprocal_length_dto: ReciprocalLengthDto):
        """
        Obtain a new instance of ReciprocalLength from a DTO unit object.

        :param reciprocal_length_dto: The ReciprocalLength DTO representation.
        :type reciprocal_length_dto: ReciprocalLengthDto
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(reciprocal_length_dto.value, reciprocal_length_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ReciprocalLength from a DTO unit json representation.

        :param data: The ReciprocalLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "InverseMeter"}
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength.from_dto(ReciprocalLengthDto.from_json(data))

    def __convert_from_base(self, from_unit: ReciprocalLengthUnits) -> float:
        value = self._value
        
        if from_unit == ReciprocalLengthUnits.InverseMeter:
            return (value)
        
        if from_unit == ReciprocalLengthUnits.InverseCentimeter:
            return (value / 1e2)
        
        if from_unit == ReciprocalLengthUnits.InverseMillimeter:
            return (value / 1e3)
        
        if from_unit == ReciprocalLengthUnits.InverseMile:
            return (value * 1609.344)
        
        if from_unit == ReciprocalLengthUnits.InverseYard:
            return (value * 0.9144)
        
        if from_unit == ReciprocalLengthUnits.InverseFoot:
            return (value * 0.3048)
        
        if from_unit == ReciprocalLengthUnits.InverseUsSurveyFoot:
            return (value * 1200 / 3937)
        
        if from_unit == ReciprocalLengthUnits.InverseInch:
            return (value * 2.54e-2)
        
        if from_unit == ReciprocalLengthUnits.InverseMil:
            return (value * 2.54e-5)
        
        if from_unit == ReciprocalLengthUnits.InverseMicroinch:
            return (value * 2.54e-8)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReciprocalLengthUnits) -> float:
        
        if to_unit == ReciprocalLengthUnits.InverseMeter:
            return (value)
        
        if to_unit == ReciprocalLengthUnits.InverseCentimeter:
            return (value * 1e2)
        
        if to_unit == ReciprocalLengthUnits.InverseMillimeter:
            return (value * 1e3)
        
        if to_unit == ReciprocalLengthUnits.InverseMile:
            return (value / 1609.344)
        
        if to_unit == ReciprocalLengthUnits.InverseYard:
            return (value / 0.9144)
        
        if to_unit == ReciprocalLengthUnits.InverseFoot:
            return (value / 0.3048)
        
        if to_unit == ReciprocalLengthUnits.InverseUsSurveyFoot:
            return (value * 3937 / 1200)
        
        if to_unit == ReciprocalLengthUnits.InverseInch:
            return (value / 2.54e-2)
        
        if to_unit == ReciprocalLengthUnits.InverseMil:
            return (value / 2.54e-5)
        
        if to_unit == ReciprocalLengthUnits.InverseMicroinch:
            return (value / 2.54e-8)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_inverse_meters(inverse_meters: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_meters.

        

        :param meters: The ReciprocalLength value in inverse_meters.
        :type inverse_meters: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_meters, ReciprocalLengthUnits.InverseMeter)

    
    @staticmethod
    def from_inverse_centimeters(inverse_centimeters: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_centimeters.

        

        :param meters: The ReciprocalLength value in inverse_centimeters.
        :type inverse_centimeters: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_centimeters, ReciprocalLengthUnits.InverseCentimeter)

    
    @staticmethod
    def from_inverse_millimeters(inverse_millimeters: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_millimeters.

        

        :param meters: The ReciprocalLength value in inverse_millimeters.
        :type inverse_millimeters: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_millimeters, ReciprocalLengthUnits.InverseMillimeter)

    
    @staticmethod
    def from_inverse_miles(inverse_miles: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_miles.

        

        :param meters: The ReciprocalLength value in inverse_miles.
        :type inverse_miles: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_miles, ReciprocalLengthUnits.InverseMile)

    
    @staticmethod
    def from_inverse_yards(inverse_yards: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_yards.

        

        :param meters: The ReciprocalLength value in inverse_yards.
        :type inverse_yards: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_yards, ReciprocalLengthUnits.InverseYard)

    
    @staticmethod
    def from_inverse_feet(inverse_feet: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_feet.

        

        :param meters: The ReciprocalLength value in inverse_feet.
        :type inverse_feet: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_feet, ReciprocalLengthUnits.InverseFoot)

    
    @staticmethod
    def from_inverse_us_survey_feet(inverse_us_survey_feet: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_us_survey_feet.

        

        :param meters: The ReciprocalLength value in inverse_us_survey_feet.
        :type inverse_us_survey_feet: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_us_survey_feet, ReciprocalLengthUnits.InverseUsSurveyFoot)

    
    @staticmethod
    def from_inverse_inches(inverse_inches: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_inches.

        

        :param meters: The ReciprocalLength value in inverse_inches.
        :type inverse_inches: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_inches, ReciprocalLengthUnits.InverseInch)

    
    @staticmethod
    def from_inverse_mils(inverse_mils: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_mils.

        

        :param meters: The ReciprocalLength value in inverse_mils.
        :type inverse_mils: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_mils, ReciprocalLengthUnits.InverseMil)

    
    @staticmethod
    def from_inverse_microinches(inverse_microinches: float):
        """
        Create a new instance of ReciprocalLength from a value in inverse_microinches.

        

        :param meters: The ReciprocalLength value in inverse_microinches.
        :type inverse_microinches: float
        :return: A new instance of ReciprocalLength.
        :rtype: ReciprocalLength
        """
        return ReciprocalLength(inverse_microinches, ReciprocalLengthUnits.InverseMicroinch)

    
    @property
    def inverse_meters(self) -> float:
        """
        
        """
        if self.__inverse_meters != None:
            return self.__inverse_meters
        self.__inverse_meters = self.__convert_from_base(ReciprocalLengthUnits.InverseMeter)
        return self.__inverse_meters

    
    @property
    def inverse_centimeters(self) -> float:
        """
        
        """
        if self.__inverse_centimeters != None:
            return self.__inverse_centimeters
        self.__inverse_centimeters = self.__convert_from_base(ReciprocalLengthUnits.InverseCentimeter)
        return self.__inverse_centimeters

    
    @property
    def inverse_millimeters(self) -> float:
        """
        
        """
        if self.__inverse_millimeters != None:
            return self.__inverse_millimeters
        self.__inverse_millimeters = self.__convert_from_base(ReciprocalLengthUnits.InverseMillimeter)
        return self.__inverse_millimeters

    
    @property
    def inverse_miles(self) -> float:
        """
        
        """
        if self.__inverse_miles != None:
            return self.__inverse_miles
        self.__inverse_miles = self.__convert_from_base(ReciprocalLengthUnits.InverseMile)
        return self.__inverse_miles

    
    @property
    def inverse_yards(self) -> float:
        """
        
        """
        if self.__inverse_yards != None:
            return self.__inverse_yards
        self.__inverse_yards = self.__convert_from_base(ReciprocalLengthUnits.InverseYard)
        return self.__inverse_yards

    
    @property
    def inverse_feet(self) -> float:
        """
        
        """
        if self.__inverse_feet != None:
            return self.__inverse_feet
        self.__inverse_feet = self.__convert_from_base(ReciprocalLengthUnits.InverseFoot)
        return self.__inverse_feet

    
    @property
    def inverse_us_survey_feet(self) -> float:
        """
        
        """
        if self.__inverse_us_survey_feet != None:
            return self.__inverse_us_survey_feet
        self.__inverse_us_survey_feet = self.__convert_from_base(ReciprocalLengthUnits.InverseUsSurveyFoot)
        return self.__inverse_us_survey_feet

    
    @property
    def inverse_inches(self) -> float:
        """
        
        """
        if self.__inverse_inches != None:
            return self.__inverse_inches
        self.__inverse_inches = self.__convert_from_base(ReciprocalLengthUnits.InverseInch)
        return self.__inverse_inches

    
    @property
    def inverse_mils(self) -> float:
        """
        
        """
        if self.__inverse_mils != None:
            return self.__inverse_mils
        self.__inverse_mils = self.__convert_from_base(ReciprocalLengthUnits.InverseMil)
        return self.__inverse_mils

    
    @property
    def inverse_microinches(self) -> float:
        """
        
        """
        if self.__inverse_microinches != None:
            return self.__inverse_microinches
        self.__inverse_microinches = self.__convert_from_base(ReciprocalLengthUnits.InverseMicroinch)
        return self.__inverse_microinches

    
    def to_string(self, unit: ReciprocalLengthUnits = ReciprocalLengthUnits.InverseMeter, fractional_digits: int = None) -> str:
        """
        Format the ReciprocalLength to a string.
        
        Note: the default format for ReciprocalLength is InverseMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ReciprocalLength. Default is 'InverseMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ReciprocalLengthUnits.InverseMeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_meters, fractional_digits)} m⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseCentimeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_centimeters, fractional_digits)} cm⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseMillimeter:
            return f"""{super()._truncate_fraction_digits(self.inverse_millimeters, fractional_digits)} mm⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseMile:
            return f"""{super()._truncate_fraction_digits(self.inverse_miles, fractional_digits)} mi⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseYard:
            return f"""{super()._truncate_fraction_digits(self.inverse_yards, fractional_digits)} yd⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseFoot:
            return f"""{super()._truncate_fraction_digits(self.inverse_feet, fractional_digits)} ft⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseUsSurveyFoot:
            return f"""{super()._truncate_fraction_digits(self.inverse_us_survey_feet, fractional_digits)} ftUS⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseInch:
            return f"""{super()._truncate_fraction_digits(self.inverse_inches, fractional_digits)} in⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseMil:
            return f"""{super()._truncate_fraction_digits(self.inverse_mils, fractional_digits)} mil⁻¹"""
        
        if unit == ReciprocalLengthUnits.InverseMicroinch:
            return f"""{super()._truncate_fraction_digits(self.inverse_microinches, fractional_digits)} µin⁻¹"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReciprocalLengthUnits = ReciprocalLengthUnits.InverseMeter) -> str:
        """
        Get ReciprocalLength unit abbreviation.
        Note! the default abbreviation for ReciprocalLength is InverseMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseMeter:
            return """m⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseCentimeter:
            return """cm⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseMillimeter:
            return """mm⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseMile:
            return """mi⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseYard:
            return """yd⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseFoot:
            return """ft⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseUsSurveyFoot:
            return """ftUS⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseInch:
            return """in⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseMil:
            return """mil⁻¹"""
        
        if unit_abbreviation == ReciprocalLengthUnits.InverseMicroinch:
            return """µin⁻¹"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class CoefficientOfThermalExpansionUnits(Enum):
        """
            CoefficientOfThermalExpansionUnits enumeration
        """
        
        PerKelvin = 'PerKelvin'
        """
            
        """
        
        PerDegreeCelsius = 'PerDegreeCelsius'
        """
            
        """
        
        PerDegreeFahrenheit = 'PerDegreeFahrenheit'
        """
            
        """
        
        PpmPerKelvin = 'PpmPerKelvin'
        """
            
        """
        
        PpmPerDegreeCelsius = 'PpmPerDegreeCelsius'
        """
            
        """
        
        PpmPerDegreeFahrenheit = 'PpmPerDegreeFahrenheit'
        """
            
        """
        

class CoefficientOfThermalExpansionDto:
    """
    A DTO representation of a CoefficientOfThermalExpansion

    Attributes:
        value (float): The value of the CoefficientOfThermalExpansion.
        unit (CoefficientOfThermalExpansionUnits): The specific unit that the CoefficientOfThermalExpansion value is representing.
    """

    def __init__(self, value: float, unit: CoefficientOfThermalExpansionUnits):
        """
        Create a new DTO representation of a CoefficientOfThermalExpansion

        Parameters:
            value (float): The value of the CoefficientOfThermalExpansion.
            unit (CoefficientOfThermalExpansionUnits): The specific unit that the CoefficientOfThermalExpansion value is representing.
        """
        self.value: float = value
        """
        The value of the CoefficientOfThermalExpansion
        """
        self.unit: CoefficientOfThermalExpansionUnits = unit
        """
        The specific unit that the CoefficientOfThermalExpansion value is representing
        """

    def to_json(self):
        """
        Get a CoefficientOfThermalExpansion DTO JSON object representing the current unit.

        :return: JSON object represents CoefficientOfThermalExpansion DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "PerKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of CoefficientOfThermalExpansion DTO from a json representation.

        :param data: The CoefficientOfThermalExpansion DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "PerKelvin"}
        :return: A new instance of CoefficientOfThermalExpansionDto.
        :rtype: CoefficientOfThermalExpansionDto
        """
        return CoefficientOfThermalExpansionDto(value=data["value"], unit=CoefficientOfThermalExpansionUnits(data["unit"]))


class CoefficientOfThermalExpansion(AbstractMeasure):
    """
    A unit that represents a fractional change in size in response to a change in temperature.

    Args:
        value (float): The value.
        from_unit (CoefficientOfThermalExpansionUnits): The CoefficientOfThermalExpansion unit to create from, The default unit is PerKelvin
    """
    def __init__(self, value: float, from_unit: CoefficientOfThermalExpansionUnits = CoefficientOfThermalExpansionUnits.PerKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__per_kelvin = None
        
        self.__per_degree_celsius = None
        
        self.__per_degree_fahrenheit = None
        
        self.__ppm_per_kelvin = None
        
        self.__ppm_per_degree_celsius = None
        
        self.__ppm_per_degree_fahrenheit = None
        

    def convert(self, unit: CoefficientOfThermalExpansionUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: CoefficientOfThermalExpansionUnits = CoefficientOfThermalExpansionUnits.PerKelvin) -> CoefficientOfThermalExpansionDto:
        """
        Get a new instance of CoefficientOfThermalExpansion DTO representing the current unit.

        :param hold_in_unit: The specific CoefficientOfThermalExpansion unit to store the CoefficientOfThermalExpansion value in the DTO representation.
        :type hold_in_unit: CoefficientOfThermalExpansionUnits
        :return: A new instance of CoefficientOfThermalExpansionDto.
        :rtype: CoefficientOfThermalExpansionDto
        """
        return CoefficientOfThermalExpansionDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: CoefficientOfThermalExpansionUnits = CoefficientOfThermalExpansionUnits.PerKelvin):
        """
        Get a CoefficientOfThermalExpansion DTO JSON object representing the current unit.

        :param hold_in_unit: The specific CoefficientOfThermalExpansion unit to store the CoefficientOfThermalExpansion value in the DTO representation.
        :type hold_in_unit: CoefficientOfThermalExpansionUnits
        :return: JSON object represents CoefficientOfThermalExpansion DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "PerKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(coefficient_of_thermal_expansion_dto: CoefficientOfThermalExpansionDto):
        """
        Obtain a new instance of CoefficientOfThermalExpansion from a DTO unit object.

        :param coefficient_of_thermal_expansion_dto: The CoefficientOfThermalExpansion DTO representation.
        :type coefficient_of_thermal_expansion_dto: CoefficientOfThermalExpansionDto
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(coefficient_of_thermal_expansion_dto.value, coefficient_of_thermal_expansion_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of CoefficientOfThermalExpansion from a DTO unit json representation.

        :param data: The CoefficientOfThermalExpansion DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "PerKelvin"}
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion.from_dto(CoefficientOfThermalExpansionDto.from_json(data))

    def __convert_from_base(self, from_unit: CoefficientOfThermalExpansionUnits) -> float:
        value = self._value
        
        if from_unit == CoefficientOfThermalExpansionUnits.PerKelvin:
            return (value)
        
        if from_unit == CoefficientOfThermalExpansionUnits.PerDegreeCelsius:
            return (value)
        
        if from_unit == CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit:
            return (value * 5 / 9)
        
        if from_unit == CoefficientOfThermalExpansionUnits.PpmPerKelvin:
            return (value * 1e6)
        
        if from_unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius:
            return (value * 1e6)
        
        if from_unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit:
            return (value * 5e6 / 9)
        
        return None


    def __convert_to_base(self, value: float, to_unit: CoefficientOfThermalExpansionUnits) -> float:
        
        if to_unit == CoefficientOfThermalExpansionUnits.PerKelvin:
            return (value)
        
        if to_unit == CoefficientOfThermalExpansionUnits.PerDegreeCelsius:
            return (value)
        
        if to_unit == CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit:
            return (value * 9 / 5)
        
        if to_unit == CoefficientOfThermalExpansionUnits.PpmPerKelvin:
            return (value / 1e6)
        
        if to_unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius:
            return (value / 1e6)
        
        if to_unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit:
            return (value * 9 / 5e6)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_per_kelvin(per_kelvin: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in per_kelvin.

        

        :param meters: The CoefficientOfThermalExpansion value in per_kelvin.
        :type per_kelvin: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(per_kelvin, CoefficientOfThermalExpansionUnits.PerKelvin)

    
    @staticmethod
    def from_per_degree_celsius(per_degree_celsius: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in per_degree_celsius.

        

        :param meters: The CoefficientOfThermalExpansion value in per_degree_celsius.
        :type per_degree_celsius: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(per_degree_celsius, CoefficientOfThermalExpansionUnits.PerDegreeCelsius)

    
    @staticmethod
    def from_per_degree_fahrenheit(per_degree_fahrenheit: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in per_degree_fahrenheit.

        

        :param meters: The CoefficientOfThermalExpansion value in per_degree_fahrenheit.
        :type per_degree_fahrenheit: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(per_degree_fahrenheit, CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit)

    
    @staticmethod
    def from_ppm_per_kelvin(ppm_per_kelvin: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in ppm_per_kelvin.

        

        :param meters: The CoefficientOfThermalExpansion value in ppm_per_kelvin.
        :type ppm_per_kelvin: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(ppm_per_kelvin, CoefficientOfThermalExpansionUnits.PpmPerKelvin)

    
    @staticmethod
    def from_ppm_per_degree_celsius(ppm_per_degree_celsius: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in ppm_per_degree_celsius.

        

        :param meters: The CoefficientOfThermalExpansion value in ppm_per_degree_celsius.
        :type ppm_per_degree_celsius: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(ppm_per_degree_celsius, CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius)

    
    @staticmethod
    def from_ppm_per_degree_fahrenheit(ppm_per_degree_fahrenheit: float):
        """
        Create a new instance of CoefficientOfThermalExpansion from a value in ppm_per_degree_fahrenheit.

        

        :param meters: The CoefficientOfThermalExpansion value in ppm_per_degree_fahrenheit.
        :type ppm_per_degree_fahrenheit: float
        :return: A new instance of CoefficientOfThermalExpansion.
        :rtype: CoefficientOfThermalExpansion
        """
        return CoefficientOfThermalExpansion(ppm_per_degree_fahrenheit, CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit)

    
    @property
    def per_kelvin(self) -> float:
        """
        
        """
        if self.__per_kelvin != None:
            return self.__per_kelvin
        self.__per_kelvin = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PerKelvin)
        return self.__per_kelvin

    
    @property
    def per_degree_celsius(self) -> float:
        """
        
        """
        if self.__per_degree_celsius != None:
            return self.__per_degree_celsius
        self.__per_degree_celsius = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PerDegreeCelsius)
        return self.__per_degree_celsius

    
    @property
    def per_degree_fahrenheit(self) -> float:
        """
        
        """
        if self.__per_degree_fahrenheit != None:
            return self.__per_degree_fahrenheit
        self.__per_degree_fahrenheit = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit)
        return self.__per_degree_fahrenheit

    
    @property
    def ppm_per_kelvin(self) -> float:
        """
        
        """
        if self.__ppm_per_kelvin != None:
            return self.__ppm_per_kelvin
        self.__ppm_per_kelvin = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PpmPerKelvin)
        return self.__ppm_per_kelvin

    
    @property
    def ppm_per_degree_celsius(self) -> float:
        """
        
        """
        if self.__ppm_per_degree_celsius != None:
            return self.__ppm_per_degree_celsius
        self.__ppm_per_degree_celsius = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius)
        return self.__ppm_per_degree_celsius

    
    @property
    def ppm_per_degree_fahrenheit(self) -> float:
        """
        
        """
        if self.__ppm_per_degree_fahrenheit != None:
            return self.__ppm_per_degree_fahrenheit
        self.__ppm_per_degree_fahrenheit = self.__convert_from_base(CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit)
        return self.__ppm_per_degree_fahrenheit

    
    def to_string(self, unit: CoefficientOfThermalExpansionUnits = CoefficientOfThermalExpansionUnits.PerKelvin, fractional_digits: int = None) -> str:
        """
        Format the CoefficientOfThermalExpansion to a string.
        
        Note: the default format for CoefficientOfThermalExpansion is PerKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the CoefficientOfThermalExpansion. Default is 'PerKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == CoefficientOfThermalExpansionUnits.PerKelvin:
            return f"""{super()._truncate_fraction_digits(self.per_kelvin, fractional_digits)} K⁻¹"""
        
        if unit == CoefficientOfThermalExpansionUnits.PerDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.per_degree_celsius, fractional_digits)} °C⁻¹"""
        
        if unit == CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.per_degree_fahrenheit, fractional_digits)} °F⁻¹"""
        
        if unit == CoefficientOfThermalExpansionUnits.PpmPerKelvin:
            return f"""{super()._truncate_fraction_digits(self.ppm_per_kelvin, fractional_digits)} ppm/K"""
        
        if unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.ppm_per_degree_celsius, fractional_digits)} ppm/°C"""
        
        if unit == CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.ppm_per_degree_fahrenheit, fractional_digits)} ppm/°F"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: CoefficientOfThermalExpansionUnits = CoefficientOfThermalExpansionUnits.PerKelvin) -> str:
        """
        Get CoefficientOfThermalExpansion unit abbreviation.
        Note! the default abbreviation for CoefficientOfThermalExpansion is PerKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PerKelvin:
            return """K⁻¹"""
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PerDegreeCelsius:
            return """°C⁻¹"""
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PerDegreeFahrenheit:
            return """°F⁻¹"""
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PpmPerKelvin:
            return """ppm/K"""
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PpmPerDegreeCelsius:
            return """ppm/°C"""
        
        if unit_abbreviation == CoefficientOfThermalExpansionUnits.PpmPerDegreeFahrenheit:
            return """ppm/°F"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class SolidAngleUnits(Enum):
        """
            SolidAngleUnits enumeration
        """
        
        Steradian = 'Steradian'
        """
            
        """
        

class SolidAngleDto:
    """
    A DTO representation of a SolidAngle

    Attributes:
        value (float): The value of the SolidAngle.
        unit (SolidAngleUnits): The specific unit that the SolidAngle value is representing.
    """

    def __init__(self, value: float, unit: SolidAngleUnits):
        """
        Create a new DTO representation of a SolidAngle

        Parameters:
            value (float): The value of the SolidAngle.
            unit (SolidAngleUnits): The specific unit that the SolidAngle value is representing.
        """
        self.value: float = value
        """
        The value of the SolidAngle
        """
        self.unit: SolidAngleUnits = unit
        """
        The specific unit that the SolidAngle value is representing
        """

    def to_json(self):
        """
        Get a SolidAngle DTO JSON object representing the current unit.

        :return: JSON object represents SolidAngle DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Steradian"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of SolidAngle DTO from a json representation.

        :param data: The SolidAngle DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Steradian"}
        :return: A new instance of SolidAngleDto.
        :rtype: SolidAngleDto
        """
        return SolidAngleDto(value=data["value"], unit=SolidAngleUnits(data["unit"]))


class SolidAngle(AbstractMeasure):
    """
    In geometry, a solid angle is the two-dimensional angle in three-dimensional space that an object subtends at a point.

    Args:
        value (float): The value.
        from_unit (SolidAngleUnits): The SolidAngle unit to create from, The default unit is Steradian
    """
    def __init__(self, value: float, from_unit: SolidAngleUnits = SolidAngleUnits.Steradian):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__steradians = None
        

    def convert(self, unit: SolidAngleUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: SolidAngleUnits = SolidAngleUnits.Steradian) -> SolidAngleDto:
        """
        Get a new instance of SolidAngle DTO representing the current unit.

        :param hold_in_unit: The specific SolidAngle unit to store the SolidAngle value in the DTO representation.
        :type hold_in_unit: SolidAngleUnits
        :return: A new instance of SolidAngleDto.
        :rtype: SolidAngleDto
        """
        return SolidAngleDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: SolidAngleUnits = SolidAngleUnits.Steradian):
        """
        Get a SolidAngle DTO JSON object representing the current unit.

        :param hold_in_unit: The specific SolidAngle unit to store the SolidAngle value in the DTO representation.
        :type hold_in_unit: SolidAngleUnits
        :return: JSON object represents SolidAngle DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Steradian"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(solid_angle_dto: SolidAngleDto):
        """
        Obtain a new instance of SolidAngle from a DTO unit object.

        :param solid_angle_dto: The SolidAngle DTO representation.
        :type solid_angle_dto: SolidAngleDto
        :return: A new instance of SolidAngle.
        :rtype: SolidAngle
        """
        return SolidAngle(solid_angle_dto.value, solid_angle_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of SolidAngle from a DTO unit json representation.

        :param data: The SolidAngle DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Steradian"}
        :return: A new instance of SolidAngle.
        :rtype: SolidAngle
        """
        return SolidAngle.from_dto(SolidAngleDto.from_json(data))

    def __convert_from_base(self, from_unit: SolidAngleUnits) -> float:
        value = self._value
        
        if from_unit == SolidAngleUnits.Steradian:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SolidAngleUnits) -> float:
        
        if to_unit == SolidAngleUnits.Steradian:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_steradians(steradians: float):
        """
        Create a new instance of SolidAngle from a value in steradians.

        

        :param meters: The SolidAngle value in steradians.
        :type steradians: float
        :return: A new instance of SolidAngle.
        :rtype: SolidAngle
        """
        return SolidAngle(steradians, SolidAngleUnits.Steradian)

    
    @property
    def steradians(self) -> float:
        """
        
        """
        if self.__steradians != None:
            return self.__steradians
        self.__steradians = self.__convert_from_base(SolidAngleUnits.Steradian)
        return self.__steradians

    
    def to_string(self, unit: SolidAngleUnits = SolidAngleUnits.Steradian, fractional_digits: int = None) -> str:
        """
        Format the SolidAngle to a string.
        
        Note: the default format for SolidAngle is Steradian.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the SolidAngle. Default is 'Steradian'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == SolidAngleUnits.Steradian:
            return f"""{super()._truncate_fraction_digits(self.steradians, fractional_digits)} sr"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: SolidAngleUnits = SolidAngleUnits.Steradian) -> str:
        """
        Get SolidAngle unit abbreviation.
        Note! the default abbreviation for SolidAngle is Steradian.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SolidAngleUnits.Steradian:
            return """sr"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricPotentialChangeRateUnits(Enum):
        """
            ElectricPotentialChangeRateUnits enumeration
        """
        
        VoltPerSecond = 'VoltPerSecond'
        """
            
        """
        
        VoltPerMicrosecond = 'VoltPerMicrosecond'
        """
            
        """
        
        VoltPerMinute = 'VoltPerMinute'
        """
            
        """
        
        VoltPerHour = 'VoltPerHour'
        """
            
        """
        
        MicrovoltPerSecond = 'MicrovoltPerSecond'
        """
            
        """
        
        MillivoltPerSecond = 'MillivoltPerSecond'
        """
            
        """
        
        KilovoltPerSecond = 'KilovoltPerSecond'
        """
            
        """
        
        MegavoltPerSecond = 'MegavoltPerSecond'
        """
            
        """
        
        MicrovoltPerMicrosecond = 'MicrovoltPerMicrosecond'
        """
            
        """
        
        MillivoltPerMicrosecond = 'MillivoltPerMicrosecond'
        """
            
        """
        
        KilovoltPerMicrosecond = 'KilovoltPerMicrosecond'
        """
            
        """
        
        MegavoltPerMicrosecond = 'MegavoltPerMicrosecond'
        """
            
        """
        
        MicrovoltPerMinute = 'MicrovoltPerMinute'
        """
            
        """
        
        MillivoltPerMinute = 'MillivoltPerMinute'
        """
            
        """
        
        KilovoltPerMinute = 'KilovoltPerMinute'
        """
            
        """
        
        MegavoltPerMinute = 'MegavoltPerMinute'
        """
            
        """
        
        MicrovoltPerHour = 'MicrovoltPerHour'
        """
            
        """
        
        MillivoltPerHour = 'MillivoltPerHour'
        """
            
        """
        
        KilovoltPerHour = 'KilovoltPerHour'
        """
            
        """
        
        MegavoltPerHour = 'MegavoltPerHour'
        """
            
        """
        

class ElectricPotentialChangeRateDto:
    """
    A DTO representation of a ElectricPotentialChangeRate

    Attributes:
        value (float): The value of the ElectricPotentialChangeRate.
        unit (ElectricPotentialChangeRateUnits): The specific unit that the ElectricPotentialChangeRate value is representing.
    """

    def __init__(self, value: float, unit: ElectricPotentialChangeRateUnits):
        """
        Create a new DTO representation of a ElectricPotentialChangeRate

        Parameters:
            value (float): The value of the ElectricPotentialChangeRate.
            unit (ElectricPotentialChangeRateUnits): The specific unit that the ElectricPotentialChangeRate value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricPotentialChangeRate
        """
        self.unit: ElectricPotentialChangeRateUnits = unit
        """
        The specific unit that the ElectricPotentialChangeRate value is representing
        """

    def to_json(self):
        """
        Get a ElectricPotentialChangeRate DTO JSON object representing the current unit.

        :return: JSON object represents ElectricPotentialChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricPotentialChangeRate DTO from a json representation.

        :param data: The ElectricPotentialChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltPerSecond"}
        :return: A new instance of ElectricPotentialChangeRateDto.
        :rtype: ElectricPotentialChangeRateDto
        """
        return ElectricPotentialChangeRateDto(value=data["value"], unit=ElectricPotentialChangeRateUnits(data["unit"]))


class ElectricPotentialChangeRate(AbstractMeasure):
    """
    ElectricPotential change rate is the ratio of the electric potential change to the time during which the change occurred (value of electric potential changes per unit time).

    Args:
        value (float): The value.
        from_unit (ElectricPotentialChangeRateUnits): The ElectricPotentialChangeRate unit to create from, The default unit is VoltPerSecond
    """
    def __init__(self, value: float, from_unit: ElectricPotentialChangeRateUnits = ElectricPotentialChangeRateUnits.VoltPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__volts_per_seconds = None
        
        self.__volts_per_microseconds = None
        
        self.__volts_per_minutes = None
        
        self.__volts_per_hours = None
        
        self.__microvolts_per_seconds = None
        
        self.__millivolts_per_seconds = None
        
        self.__kilovolts_per_seconds = None
        
        self.__megavolts_per_seconds = None
        
        self.__microvolts_per_microseconds = None
        
        self.__millivolts_per_microseconds = None
        
        self.__kilovolts_per_microseconds = None
        
        self.__megavolts_per_microseconds = None
        
        self.__microvolts_per_minutes = None
        
        self.__millivolts_per_minutes = None
        
        self.__kilovolts_per_minutes = None
        
        self.__megavolts_per_minutes = None
        
        self.__microvolts_per_hours = None
        
        self.__millivolts_per_hours = None
        
        self.__kilovolts_per_hours = None
        
        self.__megavolts_per_hours = None
        

    def convert(self, unit: ElectricPotentialChangeRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricPotentialChangeRateUnits = ElectricPotentialChangeRateUnits.VoltPerSecond) -> ElectricPotentialChangeRateDto:
        """
        Get a new instance of ElectricPotentialChangeRate DTO representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialChangeRate unit to store the ElectricPotentialChangeRate value in the DTO representation.
        :type hold_in_unit: ElectricPotentialChangeRateUnits
        :return: A new instance of ElectricPotentialChangeRateDto.
        :rtype: ElectricPotentialChangeRateDto
        """
        return ElectricPotentialChangeRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricPotentialChangeRateUnits = ElectricPotentialChangeRateUnits.VoltPerSecond):
        """
        Get a ElectricPotentialChangeRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialChangeRate unit to store the ElectricPotentialChangeRate value in the DTO representation.
        :type hold_in_unit: ElectricPotentialChangeRateUnits
        :return: JSON object represents ElectricPotentialChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_potential_change_rate_dto: ElectricPotentialChangeRateDto):
        """
        Obtain a new instance of ElectricPotentialChangeRate from a DTO unit object.

        :param electric_potential_change_rate_dto: The ElectricPotentialChangeRate DTO representation.
        :type electric_potential_change_rate_dto: ElectricPotentialChangeRateDto
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(electric_potential_change_rate_dto.value, electric_potential_change_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricPotentialChangeRate from a DTO unit json representation.

        :param data: The ElectricPotentialChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltPerSecond"}
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate.from_dto(ElectricPotentialChangeRateDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricPotentialChangeRateUnits) -> float:
        value = self._value
        
        if from_unit == ElectricPotentialChangeRateUnits.VoltPerSecond:
            return (value)
        
        if from_unit == ElectricPotentialChangeRateUnits.VoltPerMicrosecond:
            return (value / 1e6)
        
        if from_unit == ElectricPotentialChangeRateUnits.VoltPerMinute:
            return (value * 60)
        
        if from_unit == ElectricPotentialChangeRateUnits.VoltPerHour:
            return (value * 3600)
        
        if from_unit == ElectricPotentialChangeRateUnits.MicrovoltPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialChangeRateUnits.MillivoltPerSecond:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialChangeRateUnits.KilovoltPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MegavoltPerSecond:
            return ((value) / 1000000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond:
            return ((value / 1e6) / 1e-06)
        
        if from_unit == ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond:
            return ((value / 1e6) / 0.001)
        
        if from_unit == ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond:
            return ((value / 1e6) / 1000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond:
            return ((value / 1e6) / 1000000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MicrovoltPerMinute:
            return ((value * 60) / 1e-06)
        
        if from_unit == ElectricPotentialChangeRateUnits.MillivoltPerMinute:
            return ((value * 60) / 0.001)
        
        if from_unit == ElectricPotentialChangeRateUnits.KilovoltPerMinute:
            return ((value * 60) / 1000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MegavoltPerMinute:
            return ((value * 60) / 1000000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MicrovoltPerHour:
            return ((value * 3600) / 1e-06)
        
        if from_unit == ElectricPotentialChangeRateUnits.MillivoltPerHour:
            return ((value * 3600) / 0.001)
        
        if from_unit == ElectricPotentialChangeRateUnits.KilovoltPerHour:
            return ((value * 3600) / 1000.0)
        
        if from_unit == ElectricPotentialChangeRateUnits.MegavoltPerHour:
            return ((value * 3600) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialChangeRateUnits) -> float:
        
        if to_unit == ElectricPotentialChangeRateUnits.VoltPerSecond:
            return (value)
        
        if to_unit == ElectricPotentialChangeRateUnits.VoltPerMicrosecond:
            return (value * 1e6)
        
        if to_unit == ElectricPotentialChangeRateUnits.VoltPerMinute:
            return (value / 60)
        
        if to_unit == ElectricPotentialChangeRateUnits.VoltPerHour:
            return (value / 3600)
        
        if to_unit == ElectricPotentialChangeRateUnits.MicrovoltPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialChangeRateUnits.MillivoltPerSecond:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialChangeRateUnits.KilovoltPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MegavoltPerSecond:
            return ((value) * 1000000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond:
            return ((value * 1e6) * 1e-06)
        
        if to_unit == ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond:
            return ((value * 1e6) * 0.001)
        
        if to_unit == ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond:
            return ((value * 1e6) * 1000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond:
            return ((value * 1e6) * 1000000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MicrovoltPerMinute:
            return ((value / 60) * 1e-06)
        
        if to_unit == ElectricPotentialChangeRateUnits.MillivoltPerMinute:
            return ((value / 60) * 0.001)
        
        if to_unit == ElectricPotentialChangeRateUnits.KilovoltPerMinute:
            return ((value / 60) * 1000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MegavoltPerMinute:
            return ((value / 60) * 1000000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MicrovoltPerHour:
            return ((value / 3600) * 1e-06)
        
        if to_unit == ElectricPotentialChangeRateUnits.MillivoltPerHour:
            return ((value / 3600) * 0.001)
        
        if to_unit == ElectricPotentialChangeRateUnits.KilovoltPerHour:
            return ((value / 3600) * 1000.0)
        
        if to_unit == ElectricPotentialChangeRateUnits.MegavoltPerHour:
            return ((value / 3600) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_volts_per_seconds(volts_per_seconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in volts_per_seconds.

        

        :param meters: The ElectricPotentialChangeRate value in volts_per_seconds.
        :type volts_per_seconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(volts_per_seconds, ElectricPotentialChangeRateUnits.VoltPerSecond)

    
    @staticmethod
    def from_volts_per_microseconds(volts_per_microseconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in volts_per_microseconds.

        

        :param meters: The ElectricPotentialChangeRate value in volts_per_microseconds.
        :type volts_per_microseconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(volts_per_microseconds, ElectricPotentialChangeRateUnits.VoltPerMicrosecond)

    
    @staticmethod
    def from_volts_per_minutes(volts_per_minutes: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in volts_per_minutes.

        

        :param meters: The ElectricPotentialChangeRate value in volts_per_minutes.
        :type volts_per_minutes: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(volts_per_minutes, ElectricPotentialChangeRateUnits.VoltPerMinute)

    
    @staticmethod
    def from_volts_per_hours(volts_per_hours: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in volts_per_hours.

        

        :param meters: The ElectricPotentialChangeRate value in volts_per_hours.
        :type volts_per_hours: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(volts_per_hours, ElectricPotentialChangeRateUnits.VoltPerHour)

    
    @staticmethod
    def from_microvolts_per_seconds(microvolts_per_seconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in microvolts_per_seconds.

        

        :param meters: The ElectricPotentialChangeRate value in microvolts_per_seconds.
        :type microvolts_per_seconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(microvolts_per_seconds, ElectricPotentialChangeRateUnits.MicrovoltPerSecond)

    
    @staticmethod
    def from_millivolts_per_seconds(millivolts_per_seconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in millivolts_per_seconds.

        

        :param meters: The ElectricPotentialChangeRate value in millivolts_per_seconds.
        :type millivolts_per_seconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(millivolts_per_seconds, ElectricPotentialChangeRateUnits.MillivoltPerSecond)

    
    @staticmethod
    def from_kilovolts_per_seconds(kilovolts_per_seconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in kilovolts_per_seconds.

        

        :param meters: The ElectricPotentialChangeRate value in kilovolts_per_seconds.
        :type kilovolts_per_seconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(kilovolts_per_seconds, ElectricPotentialChangeRateUnits.KilovoltPerSecond)

    
    @staticmethod
    def from_megavolts_per_seconds(megavolts_per_seconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in megavolts_per_seconds.

        

        :param meters: The ElectricPotentialChangeRate value in megavolts_per_seconds.
        :type megavolts_per_seconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(megavolts_per_seconds, ElectricPotentialChangeRateUnits.MegavoltPerSecond)

    
    @staticmethod
    def from_microvolts_per_microseconds(microvolts_per_microseconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in microvolts_per_microseconds.

        

        :param meters: The ElectricPotentialChangeRate value in microvolts_per_microseconds.
        :type microvolts_per_microseconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(microvolts_per_microseconds, ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond)

    
    @staticmethod
    def from_millivolts_per_microseconds(millivolts_per_microseconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in millivolts_per_microseconds.

        

        :param meters: The ElectricPotentialChangeRate value in millivolts_per_microseconds.
        :type millivolts_per_microseconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(millivolts_per_microseconds, ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond)

    
    @staticmethod
    def from_kilovolts_per_microseconds(kilovolts_per_microseconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in kilovolts_per_microseconds.

        

        :param meters: The ElectricPotentialChangeRate value in kilovolts_per_microseconds.
        :type kilovolts_per_microseconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(kilovolts_per_microseconds, ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond)

    
    @staticmethod
    def from_megavolts_per_microseconds(megavolts_per_microseconds: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in megavolts_per_microseconds.

        

        :param meters: The ElectricPotentialChangeRate value in megavolts_per_microseconds.
        :type megavolts_per_microseconds: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(megavolts_per_microseconds, ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond)

    
    @staticmethod
    def from_microvolts_per_minutes(microvolts_per_minutes: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in microvolts_per_minutes.

        

        :param meters: The ElectricPotentialChangeRate value in microvolts_per_minutes.
        :type microvolts_per_minutes: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(microvolts_per_minutes, ElectricPotentialChangeRateUnits.MicrovoltPerMinute)

    
    @staticmethod
    def from_millivolts_per_minutes(millivolts_per_minutes: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in millivolts_per_minutes.

        

        :param meters: The ElectricPotentialChangeRate value in millivolts_per_minutes.
        :type millivolts_per_minutes: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(millivolts_per_minutes, ElectricPotentialChangeRateUnits.MillivoltPerMinute)

    
    @staticmethod
    def from_kilovolts_per_minutes(kilovolts_per_minutes: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in kilovolts_per_minutes.

        

        :param meters: The ElectricPotentialChangeRate value in kilovolts_per_minutes.
        :type kilovolts_per_minutes: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(kilovolts_per_minutes, ElectricPotentialChangeRateUnits.KilovoltPerMinute)

    
    @staticmethod
    def from_megavolts_per_minutes(megavolts_per_minutes: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in megavolts_per_minutes.

        

        :param meters: The ElectricPotentialChangeRate value in megavolts_per_minutes.
        :type megavolts_per_minutes: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(megavolts_per_minutes, ElectricPotentialChangeRateUnits.MegavoltPerMinute)

    
    @staticmethod
    def from_microvolts_per_hours(microvolts_per_hours: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in microvolts_per_hours.

        

        :param meters: The ElectricPotentialChangeRate value in microvolts_per_hours.
        :type microvolts_per_hours: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(microvolts_per_hours, ElectricPotentialChangeRateUnits.MicrovoltPerHour)

    
    @staticmethod
    def from_millivolts_per_hours(millivolts_per_hours: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in millivolts_per_hours.

        

        :param meters: The ElectricPotentialChangeRate value in millivolts_per_hours.
        :type millivolts_per_hours: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(millivolts_per_hours, ElectricPotentialChangeRateUnits.MillivoltPerHour)

    
    @staticmethod
    def from_kilovolts_per_hours(kilovolts_per_hours: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in kilovolts_per_hours.

        

        :param meters: The ElectricPotentialChangeRate value in kilovolts_per_hours.
        :type kilovolts_per_hours: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(kilovolts_per_hours, ElectricPotentialChangeRateUnits.KilovoltPerHour)

    
    @staticmethod
    def from_megavolts_per_hours(megavolts_per_hours: float):
        """
        Create a new instance of ElectricPotentialChangeRate from a value in megavolts_per_hours.

        

        :param meters: The ElectricPotentialChangeRate value in megavolts_per_hours.
        :type megavolts_per_hours: float
        :return: A new instance of ElectricPotentialChangeRate.
        :rtype: ElectricPotentialChangeRate
        """
        return ElectricPotentialChangeRate(megavolts_per_hours, ElectricPotentialChangeRateUnits.MegavoltPerHour)

    
    @property
    def volts_per_seconds(self) -> float:
        """
        
        """
        if self.__volts_per_seconds != None:
            return self.__volts_per_seconds
        self.__volts_per_seconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.VoltPerSecond)
        return self.__volts_per_seconds

    
    @property
    def volts_per_microseconds(self) -> float:
        """
        
        """
        if self.__volts_per_microseconds != None:
            return self.__volts_per_microseconds
        self.__volts_per_microseconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.VoltPerMicrosecond)
        return self.__volts_per_microseconds

    
    @property
    def volts_per_minutes(self) -> float:
        """
        
        """
        if self.__volts_per_minutes != None:
            return self.__volts_per_minutes
        self.__volts_per_minutes = self.__convert_from_base(ElectricPotentialChangeRateUnits.VoltPerMinute)
        return self.__volts_per_minutes

    
    @property
    def volts_per_hours(self) -> float:
        """
        
        """
        if self.__volts_per_hours != None:
            return self.__volts_per_hours
        self.__volts_per_hours = self.__convert_from_base(ElectricPotentialChangeRateUnits.VoltPerHour)
        return self.__volts_per_hours

    
    @property
    def microvolts_per_seconds(self) -> float:
        """
        
        """
        if self.__microvolts_per_seconds != None:
            return self.__microvolts_per_seconds
        self.__microvolts_per_seconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MicrovoltPerSecond)
        return self.__microvolts_per_seconds

    
    @property
    def millivolts_per_seconds(self) -> float:
        """
        
        """
        if self.__millivolts_per_seconds != None:
            return self.__millivolts_per_seconds
        self.__millivolts_per_seconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MillivoltPerSecond)
        return self.__millivolts_per_seconds

    
    @property
    def kilovolts_per_seconds(self) -> float:
        """
        
        """
        if self.__kilovolts_per_seconds != None:
            return self.__kilovolts_per_seconds
        self.__kilovolts_per_seconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.KilovoltPerSecond)
        return self.__kilovolts_per_seconds

    
    @property
    def megavolts_per_seconds(self) -> float:
        """
        
        """
        if self.__megavolts_per_seconds != None:
            return self.__megavolts_per_seconds
        self.__megavolts_per_seconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MegavoltPerSecond)
        return self.__megavolts_per_seconds

    
    @property
    def microvolts_per_microseconds(self) -> float:
        """
        
        """
        if self.__microvolts_per_microseconds != None:
            return self.__microvolts_per_microseconds
        self.__microvolts_per_microseconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond)
        return self.__microvolts_per_microseconds

    
    @property
    def millivolts_per_microseconds(self) -> float:
        """
        
        """
        if self.__millivolts_per_microseconds != None:
            return self.__millivolts_per_microseconds
        self.__millivolts_per_microseconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond)
        return self.__millivolts_per_microseconds

    
    @property
    def kilovolts_per_microseconds(self) -> float:
        """
        
        """
        if self.__kilovolts_per_microseconds != None:
            return self.__kilovolts_per_microseconds
        self.__kilovolts_per_microseconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond)
        return self.__kilovolts_per_microseconds

    
    @property
    def megavolts_per_microseconds(self) -> float:
        """
        
        """
        if self.__megavolts_per_microseconds != None:
            return self.__megavolts_per_microseconds
        self.__megavolts_per_microseconds = self.__convert_from_base(ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond)
        return self.__megavolts_per_microseconds

    
    @property
    def microvolts_per_minutes(self) -> float:
        """
        
        """
        if self.__microvolts_per_minutes != None:
            return self.__microvolts_per_minutes
        self.__microvolts_per_minutes = self.__convert_from_base(ElectricPotentialChangeRateUnits.MicrovoltPerMinute)
        return self.__microvolts_per_minutes

    
    @property
    def millivolts_per_minutes(self) -> float:
        """
        
        """
        if self.__millivolts_per_minutes != None:
            return self.__millivolts_per_minutes
        self.__millivolts_per_minutes = self.__convert_from_base(ElectricPotentialChangeRateUnits.MillivoltPerMinute)
        return self.__millivolts_per_minutes

    
    @property
    def kilovolts_per_minutes(self) -> float:
        """
        
        """
        if self.__kilovolts_per_minutes != None:
            return self.__kilovolts_per_minutes
        self.__kilovolts_per_minutes = self.__convert_from_base(ElectricPotentialChangeRateUnits.KilovoltPerMinute)
        return self.__kilovolts_per_minutes

    
    @property
    def megavolts_per_minutes(self) -> float:
        """
        
        """
        if self.__megavolts_per_minutes != None:
            return self.__megavolts_per_minutes
        self.__megavolts_per_minutes = self.__convert_from_base(ElectricPotentialChangeRateUnits.MegavoltPerMinute)
        return self.__megavolts_per_minutes

    
    @property
    def microvolts_per_hours(self) -> float:
        """
        
        """
        if self.__microvolts_per_hours != None:
            return self.__microvolts_per_hours
        self.__microvolts_per_hours = self.__convert_from_base(ElectricPotentialChangeRateUnits.MicrovoltPerHour)
        return self.__microvolts_per_hours

    
    @property
    def millivolts_per_hours(self) -> float:
        """
        
        """
        if self.__millivolts_per_hours != None:
            return self.__millivolts_per_hours
        self.__millivolts_per_hours = self.__convert_from_base(ElectricPotentialChangeRateUnits.MillivoltPerHour)
        return self.__millivolts_per_hours

    
    @property
    def kilovolts_per_hours(self) -> float:
        """
        
        """
        if self.__kilovolts_per_hours != None:
            return self.__kilovolts_per_hours
        self.__kilovolts_per_hours = self.__convert_from_base(ElectricPotentialChangeRateUnits.KilovoltPerHour)
        return self.__kilovolts_per_hours

    
    @property
    def megavolts_per_hours(self) -> float:
        """
        
        """
        if self.__megavolts_per_hours != None:
            return self.__megavolts_per_hours
        self.__megavolts_per_hours = self.__convert_from_base(ElectricPotentialChangeRateUnits.MegavoltPerHour)
        return self.__megavolts_per_hours

    
    def to_string(self, unit: ElectricPotentialChangeRateUnits = ElectricPotentialChangeRateUnits.VoltPerSecond, fractional_digits: int = None) -> str:
        """
        Format the ElectricPotentialChangeRate to a string.
        
        Note: the default format for ElectricPotentialChangeRate is VoltPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricPotentialChangeRate. Default is 'VoltPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricPotentialChangeRateUnits.VoltPerSecond:
            return f"""{super()._truncate_fraction_digits(self.volts_per_seconds, fractional_digits)} V/s"""
        
        if unit == ElectricPotentialChangeRateUnits.VoltPerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.volts_per_microseconds, fractional_digits)} V/μs"""
        
        if unit == ElectricPotentialChangeRateUnits.VoltPerMinute:
            return f"""{super()._truncate_fraction_digits(self.volts_per_minutes, fractional_digits)} V/min"""
        
        if unit == ElectricPotentialChangeRateUnits.VoltPerHour:
            return f"""{super()._truncate_fraction_digits(self.volts_per_hours, fractional_digits)} V/h"""
        
        if unit == ElectricPotentialChangeRateUnits.MicrovoltPerSecond:
            return f"""{super()._truncate_fraction_digits(self.microvolts_per_seconds, fractional_digits)} μV/s"""
        
        if unit == ElectricPotentialChangeRateUnits.MillivoltPerSecond:
            return f"""{super()._truncate_fraction_digits(self.millivolts_per_seconds, fractional_digits)} mV/s"""
        
        if unit == ElectricPotentialChangeRateUnits.KilovoltPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_per_seconds, fractional_digits)} kV/s"""
        
        if unit == ElectricPotentialChangeRateUnits.MegavoltPerSecond:
            return f"""{super()._truncate_fraction_digits(self.megavolts_per_seconds, fractional_digits)} MV/s"""
        
        if unit == ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.microvolts_per_microseconds, fractional_digits)} μV/μs"""
        
        if unit == ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.millivolts_per_microseconds, fractional_digits)} mV/μs"""
        
        if unit == ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_per_microseconds, fractional_digits)} kV/μs"""
        
        if unit == ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.megavolts_per_microseconds, fractional_digits)} MV/μs"""
        
        if unit == ElectricPotentialChangeRateUnits.MicrovoltPerMinute:
            return f"""{super()._truncate_fraction_digits(self.microvolts_per_minutes, fractional_digits)} μV/min"""
        
        if unit == ElectricPotentialChangeRateUnits.MillivoltPerMinute:
            return f"""{super()._truncate_fraction_digits(self.millivolts_per_minutes, fractional_digits)} mV/min"""
        
        if unit == ElectricPotentialChangeRateUnits.KilovoltPerMinute:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_per_minutes, fractional_digits)} kV/min"""
        
        if unit == ElectricPotentialChangeRateUnits.MegavoltPerMinute:
            return f"""{super()._truncate_fraction_digits(self.megavolts_per_minutes, fractional_digits)} MV/min"""
        
        if unit == ElectricPotentialChangeRateUnits.MicrovoltPerHour:
            return f"""{super()._truncate_fraction_digits(self.microvolts_per_hours, fractional_digits)} μV/h"""
        
        if unit == ElectricPotentialChangeRateUnits.MillivoltPerHour:
            return f"""{super()._truncate_fraction_digits(self.millivolts_per_hours, fractional_digits)} mV/h"""
        
        if unit == ElectricPotentialChangeRateUnits.KilovoltPerHour:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_per_hours, fractional_digits)} kV/h"""
        
        if unit == ElectricPotentialChangeRateUnits.MegavoltPerHour:
            return f"""{super()._truncate_fraction_digits(self.megavolts_per_hours, fractional_digits)} MV/h"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialChangeRateUnits = ElectricPotentialChangeRateUnits.VoltPerSecond) -> str:
        """
        Get ElectricPotentialChangeRate unit abbreviation.
        Note! the default abbreviation for ElectricPotentialChangeRate is VoltPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.VoltPerSecond:
            return """V/s"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.VoltPerMicrosecond:
            return """V/μs"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.VoltPerMinute:
            return """V/min"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.VoltPerHour:
            return """V/h"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MicrovoltPerSecond:
            return """μV/s"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MillivoltPerSecond:
            return """mV/s"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.KilovoltPerSecond:
            return """kV/s"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MegavoltPerSecond:
            return """MV/s"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MicrovoltPerMicrosecond:
            return """μV/μs"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MillivoltPerMicrosecond:
            return """mV/μs"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.KilovoltPerMicrosecond:
            return """kV/μs"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MegavoltPerMicrosecond:
            return """MV/μs"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MicrovoltPerMinute:
            return """μV/min"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MillivoltPerMinute:
            return """mV/min"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.KilovoltPerMinute:
            return """kV/min"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MegavoltPerMinute:
            return """MV/min"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MicrovoltPerHour:
            return """μV/h"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MillivoltPerHour:
            return """mV/h"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.KilovoltPerHour:
            return """kV/h"""
        
        if unit_abbreviation == ElectricPotentialChangeRateUnits.MegavoltPerHour:
            return """MV/h"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricCurrentUnits(Enum):
        """
            ElectricCurrentUnits enumeration
        """
        
        Ampere = 'Ampere'
        """
            
        """
        
        Femtoampere = 'Femtoampere'
        """
            
        """
        
        Picoampere = 'Picoampere'
        """
            
        """
        
        Nanoampere = 'Nanoampere'
        """
            
        """
        
        Microampere = 'Microampere'
        """
            
        """
        
        Milliampere = 'Milliampere'
        """
            
        """
        
        Centiampere = 'Centiampere'
        """
            
        """
        
        Kiloampere = 'Kiloampere'
        """
            
        """
        
        Megaampere = 'Megaampere'
        """
            
        """
        

class ElectricCurrentDto:
    """
    A DTO representation of a ElectricCurrent

    Attributes:
        value (float): The value of the ElectricCurrent.
        unit (ElectricCurrentUnits): The specific unit that the ElectricCurrent value is representing.
    """

    def __init__(self, value: float, unit: ElectricCurrentUnits):
        """
        Create a new DTO representation of a ElectricCurrent

        Parameters:
            value (float): The value of the ElectricCurrent.
            unit (ElectricCurrentUnits): The specific unit that the ElectricCurrent value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricCurrent
        """
        self.unit: ElectricCurrentUnits = unit
        """
        The specific unit that the ElectricCurrent value is representing
        """

    def to_json(self):
        """
        Get a ElectricCurrent DTO JSON object representing the current unit.

        :return: JSON object represents ElectricCurrent DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Ampere"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricCurrent DTO from a json representation.

        :param data: The ElectricCurrent DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Ampere"}
        :return: A new instance of ElectricCurrentDto.
        :rtype: ElectricCurrentDto
        """
        return ElectricCurrentDto(value=data["value"], unit=ElectricCurrentUnits(data["unit"]))


class ElectricCurrent(AbstractMeasure):
    """
    An electric current is a flow of electric charge. In electric circuits this charge is often carried by moving electrons in a wire. It can also be carried by ions in an electrolyte, or by both ions and electrons such as in a plasma.

    Args:
        value (float): The value.
        from_unit (ElectricCurrentUnits): The ElectricCurrent unit to create from, The default unit is Ampere
    """
    def __init__(self, value: float, from_unit: ElectricCurrentUnits = ElectricCurrentUnits.Ampere):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__amperes = None
        
        self.__femtoamperes = None
        
        self.__picoamperes = None
        
        self.__nanoamperes = None
        
        self.__microamperes = None
        
        self.__milliamperes = None
        
        self.__centiamperes = None
        
        self.__kiloamperes = None
        
        self.__megaamperes = None
        

    def convert(self, unit: ElectricCurrentUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricCurrentUnits = ElectricCurrentUnits.Ampere) -> ElectricCurrentDto:
        """
        Get a new instance of ElectricCurrent DTO representing the current unit.

        :param hold_in_unit: The specific ElectricCurrent unit to store the ElectricCurrent value in the DTO representation.
        :type hold_in_unit: ElectricCurrentUnits
        :return: A new instance of ElectricCurrentDto.
        :rtype: ElectricCurrentDto
        """
        return ElectricCurrentDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricCurrentUnits = ElectricCurrentUnits.Ampere):
        """
        Get a ElectricCurrent DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricCurrent unit to store the ElectricCurrent value in the DTO representation.
        :type hold_in_unit: ElectricCurrentUnits
        :return: JSON object represents ElectricCurrent DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Ampere"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_current_dto: ElectricCurrentDto):
        """
        Obtain a new instance of ElectricCurrent from a DTO unit object.

        :param electric_current_dto: The ElectricCurrent DTO representation.
        :type electric_current_dto: ElectricCurrentDto
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(electric_current_dto.value, electric_current_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricCurrent from a DTO unit json representation.

        :param data: The ElectricCurrent DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Ampere"}
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent.from_dto(ElectricCurrentDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricCurrentUnits) -> float:
        value = self._value
        
        if from_unit == ElectricCurrentUnits.Ampere:
            return (value)
        
        if from_unit == ElectricCurrentUnits.Femtoampere:
            return ((value) / 1e-15)
        
        if from_unit == ElectricCurrentUnits.Picoampere:
            return ((value) / 1e-12)
        
        if from_unit == ElectricCurrentUnits.Nanoampere:
            return ((value) / 1e-09)
        
        if from_unit == ElectricCurrentUnits.Microampere:
            return ((value) / 1e-06)
        
        if from_unit == ElectricCurrentUnits.Milliampere:
            return ((value) / 0.001)
        
        if from_unit == ElectricCurrentUnits.Centiampere:
            return ((value) / 0.01)
        
        if from_unit == ElectricCurrentUnits.Kiloampere:
            return ((value) / 1000.0)
        
        if from_unit == ElectricCurrentUnits.Megaampere:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricCurrentUnits) -> float:
        
        if to_unit == ElectricCurrentUnits.Ampere:
            return (value)
        
        if to_unit == ElectricCurrentUnits.Femtoampere:
            return ((value) * 1e-15)
        
        if to_unit == ElectricCurrentUnits.Picoampere:
            return ((value) * 1e-12)
        
        if to_unit == ElectricCurrentUnits.Nanoampere:
            return ((value) * 1e-09)
        
        if to_unit == ElectricCurrentUnits.Microampere:
            return ((value) * 1e-06)
        
        if to_unit == ElectricCurrentUnits.Milliampere:
            return ((value) * 0.001)
        
        if to_unit == ElectricCurrentUnits.Centiampere:
            return ((value) * 0.01)
        
        if to_unit == ElectricCurrentUnits.Kiloampere:
            return ((value) * 1000.0)
        
        if to_unit == ElectricCurrentUnits.Megaampere:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_amperes(amperes: float):
        """
        Create a new instance of ElectricCurrent from a value in amperes.

        

        :param meters: The ElectricCurrent value in amperes.
        :type amperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(amperes, ElectricCurrentUnits.Ampere)

    
    @staticmethod
    def from_femtoamperes(femtoamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in femtoamperes.

        

        :param meters: The ElectricCurrent value in femtoamperes.
        :type femtoamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(femtoamperes, ElectricCurrentUnits.Femtoampere)

    
    @staticmethod
    def from_picoamperes(picoamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in picoamperes.

        

        :param meters: The ElectricCurrent value in picoamperes.
        :type picoamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(picoamperes, ElectricCurrentUnits.Picoampere)

    
    @staticmethod
    def from_nanoamperes(nanoamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in nanoamperes.

        

        :param meters: The ElectricCurrent value in nanoamperes.
        :type nanoamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(nanoamperes, ElectricCurrentUnits.Nanoampere)

    
    @staticmethod
    def from_microamperes(microamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in microamperes.

        

        :param meters: The ElectricCurrent value in microamperes.
        :type microamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(microamperes, ElectricCurrentUnits.Microampere)

    
    @staticmethod
    def from_milliamperes(milliamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in milliamperes.

        

        :param meters: The ElectricCurrent value in milliamperes.
        :type milliamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(milliamperes, ElectricCurrentUnits.Milliampere)

    
    @staticmethod
    def from_centiamperes(centiamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in centiamperes.

        

        :param meters: The ElectricCurrent value in centiamperes.
        :type centiamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(centiamperes, ElectricCurrentUnits.Centiampere)

    
    @staticmethod
    def from_kiloamperes(kiloamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in kiloamperes.

        

        :param meters: The ElectricCurrent value in kiloamperes.
        :type kiloamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(kiloamperes, ElectricCurrentUnits.Kiloampere)

    
    @staticmethod
    def from_megaamperes(megaamperes: float):
        """
        Create a new instance of ElectricCurrent from a value in megaamperes.

        

        :param meters: The ElectricCurrent value in megaamperes.
        :type megaamperes: float
        :return: A new instance of ElectricCurrent.
        :rtype: ElectricCurrent
        """
        return ElectricCurrent(megaamperes, ElectricCurrentUnits.Megaampere)

    
    @property
    def amperes(self) -> float:
        """
        
        """
        if self.__amperes != None:
            return self.__amperes
        self.__amperes = self.__convert_from_base(ElectricCurrentUnits.Ampere)
        return self.__amperes

    
    @property
    def femtoamperes(self) -> float:
        """
        
        """
        if self.__femtoamperes != None:
            return self.__femtoamperes
        self.__femtoamperes = self.__convert_from_base(ElectricCurrentUnits.Femtoampere)
        return self.__femtoamperes

    
    @property
    def picoamperes(self) -> float:
        """
        
        """
        if self.__picoamperes != None:
            return self.__picoamperes
        self.__picoamperes = self.__convert_from_base(ElectricCurrentUnits.Picoampere)
        return self.__picoamperes

    
    @property
    def nanoamperes(self) -> float:
        """
        
        """
        if self.__nanoamperes != None:
            return self.__nanoamperes
        self.__nanoamperes = self.__convert_from_base(ElectricCurrentUnits.Nanoampere)
        return self.__nanoamperes

    
    @property
    def microamperes(self) -> float:
        """
        
        """
        if self.__microamperes != None:
            return self.__microamperes
        self.__microamperes = self.__convert_from_base(ElectricCurrentUnits.Microampere)
        return self.__microamperes

    
    @property
    def milliamperes(self) -> float:
        """
        
        """
        if self.__milliamperes != None:
            return self.__milliamperes
        self.__milliamperes = self.__convert_from_base(ElectricCurrentUnits.Milliampere)
        return self.__milliamperes

    
    @property
    def centiamperes(self) -> float:
        """
        
        """
        if self.__centiamperes != None:
            return self.__centiamperes
        self.__centiamperes = self.__convert_from_base(ElectricCurrentUnits.Centiampere)
        return self.__centiamperes

    
    @property
    def kiloamperes(self) -> float:
        """
        
        """
        if self.__kiloamperes != None:
            return self.__kiloamperes
        self.__kiloamperes = self.__convert_from_base(ElectricCurrentUnits.Kiloampere)
        return self.__kiloamperes

    
    @property
    def megaamperes(self) -> float:
        """
        
        """
        if self.__megaamperes != None:
            return self.__megaamperes
        self.__megaamperes = self.__convert_from_base(ElectricCurrentUnits.Megaampere)
        return self.__megaamperes

    
    def to_string(self, unit: ElectricCurrentUnits = ElectricCurrentUnits.Ampere, fractional_digits: int = None) -> str:
        """
        Format the ElectricCurrent to a string.
        
        Note: the default format for ElectricCurrent is Ampere.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricCurrent. Default is 'Ampere'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricCurrentUnits.Ampere:
            return f"""{super()._truncate_fraction_digits(self.amperes, fractional_digits)} A"""
        
        if unit == ElectricCurrentUnits.Femtoampere:
            return f"""{super()._truncate_fraction_digits(self.femtoamperes, fractional_digits)} fA"""
        
        if unit == ElectricCurrentUnits.Picoampere:
            return f"""{super()._truncate_fraction_digits(self.picoamperes, fractional_digits)} pA"""
        
        if unit == ElectricCurrentUnits.Nanoampere:
            return f"""{super()._truncate_fraction_digits(self.nanoamperes, fractional_digits)} nA"""
        
        if unit == ElectricCurrentUnits.Microampere:
            return f"""{super()._truncate_fraction_digits(self.microamperes, fractional_digits)} μA"""
        
        if unit == ElectricCurrentUnits.Milliampere:
            return f"""{super()._truncate_fraction_digits(self.milliamperes, fractional_digits)} mA"""
        
        if unit == ElectricCurrentUnits.Centiampere:
            return f"""{super()._truncate_fraction_digits(self.centiamperes, fractional_digits)} cA"""
        
        if unit == ElectricCurrentUnits.Kiloampere:
            return f"""{super()._truncate_fraction_digits(self.kiloamperes, fractional_digits)} kA"""
        
        if unit == ElectricCurrentUnits.Megaampere:
            return f"""{super()._truncate_fraction_digits(self.megaamperes, fractional_digits)} MA"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricCurrentUnits = ElectricCurrentUnits.Ampere) -> str:
        """
        Get ElectricCurrent unit abbreviation.
        Note! the default abbreviation for ElectricCurrent is Ampere.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricCurrentUnits.Ampere:
            return """A"""
        
        if unit_abbreviation == ElectricCurrentUnits.Femtoampere:
            return """fA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Picoampere:
            return """pA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Nanoampere:
            return """nA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Microampere:
            return """μA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Milliampere:
            return """mA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Centiampere:
            return """cA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Kiloampere:
            return """kA"""
        
        if unit_abbreviation == ElectricCurrentUnits.Megaampere:
            return """MA"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class WarpingMomentOfInertiaUnits(Enum):
        """
            WarpingMomentOfInertiaUnits enumeration
        """
        
        MeterToTheSixth = 'MeterToTheSixth'
        """
            
        """
        
        DecimeterToTheSixth = 'DecimeterToTheSixth'
        """
            
        """
        
        CentimeterToTheSixth = 'CentimeterToTheSixth'
        """
            
        """
        
        MillimeterToTheSixth = 'MillimeterToTheSixth'
        """
            
        """
        
        FootToTheSixth = 'FootToTheSixth'
        """
            
        """
        
        InchToTheSixth = 'InchToTheSixth'
        """
            
        """
        

class WarpingMomentOfInertiaDto:
    """
    A DTO representation of a WarpingMomentOfInertia

    Attributes:
        value (float): The value of the WarpingMomentOfInertia.
        unit (WarpingMomentOfInertiaUnits): The specific unit that the WarpingMomentOfInertia value is representing.
    """

    def __init__(self, value: float, unit: WarpingMomentOfInertiaUnits):
        """
        Create a new DTO representation of a WarpingMomentOfInertia

        Parameters:
            value (float): The value of the WarpingMomentOfInertia.
            unit (WarpingMomentOfInertiaUnits): The specific unit that the WarpingMomentOfInertia value is representing.
        """
        self.value: float = value
        """
        The value of the WarpingMomentOfInertia
        """
        self.unit: WarpingMomentOfInertiaUnits = unit
        """
        The specific unit that the WarpingMomentOfInertia value is representing
        """

    def to_json(self):
        """
        Get a WarpingMomentOfInertia DTO JSON object representing the current unit.

        :return: JSON object represents WarpingMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterToTheSixth"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of WarpingMomentOfInertia DTO from a json representation.

        :param data: The WarpingMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterToTheSixth"}
        :return: A new instance of WarpingMomentOfInertiaDto.
        :rtype: WarpingMomentOfInertiaDto
        """
        return WarpingMomentOfInertiaDto(value=data["value"], unit=WarpingMomentOfInertiaUnits(data["unit"]))


class WarpingMomentOfInertia(AbstractMeasure):
    """
    A geometric property of an area that is used to determine the warping stress.

    Args:
        value (float): The value.
        from_unit (WarpingMomentOfInertiaUnits): The WarpingMomentOfInertia unit to create from, The default unit is MeterToTheSixth
    """
    def __init__(self, value: float, from_unit: WarpingMomentOfInertiaUnits = WarpingMomentOfInertiaUnits.MeterToTheSixth):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__meters_to_the_sixth = None
        
        self.__decimeters_to_the_sixth = None
        
        self.__centimeters_to_the_sixth = None
        
        self.__millimeters_to_the_sixth = None
        
        self.__feet_to_the_sixth = None
        
        self.__inches_to_the_sixth = None
        

    def convert(self, unit: WarpingMomentOfInertiaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: WarpingMomentOfInertiaUnits = WarpingMomentOfInertiaUnits.MeterToTheSixth) -> WarpingMomentOfInertiaDto:
        """
        Get a new instance of WarpingMomentOfInertia DTO representing the current unit.

        :param hold_in_unit: The specific WarpingMomentOfInertia unit to store the WarpingMomentOfInertia value in the DTO representation.
        :type hold_in_unit: WarpingMomentOfInertiaUnits
        :return: A new instance of WarpingMomentOfInertiaDto.
        :rtype: WarpingMomentOfInertiaDto
        """
        return WarpingMomentOfInertiaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: WarpingMomentOfInertiaUnits = WarpingMomentOfInertiaUnits.MeterToTheSixth):
        """
        Get a WarpingMomentOfInertia DTO JSON object representing the current unit.

        :param hold_in_unit: The specific WarpingMomentOfInertia unit to store the WarpingMomentOfInertia value in the DTO representation.
        :type hold_in_unit: WarpingMomentOfInertiaUnits
        :return: JSON object represents WarpingMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterToTheSixth"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(warping_moment_of_inertia_dto: WarpingMomentOfInertiaDto):
        """
        Obtain a new instance of WarpingMomentOfInertia from a DTO unit object.

        :param warping_moment_of_inertia_dto: The WarpingMomentOfInertia DTO representation.
        :type warping_moment_of_inertia_dto: WarpingMomentOfInertiaDto
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(warping_moment_of_inertia_dto.value, warping_moment_of_inertia_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of WarpingMomentOfInertia from a DTO unit json representation.

        :param data: The WarpingMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterToTheSixth"}
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia.from_dto(WarpingMomentOfInertiaDto.from_json(data))

    def __convert_from_base(self, from_unit: WarpingMomentOfInertiaUnits) -> float:
        value = self._value
        
        if from_unit == WarpingMomentOfInertiaUnits.MeterToTheSixth:
            return (value)
        
        if from_unit == WarpingMomentOfInertiaUnits.DecimeterToTheSixth:
            return (value * 1e6)
        
        if from_unit == WarpingMomentOfInertiaUnits.CentimeterToTheSixth:
            return (value * 1e12)
        
        if from_unit == WarpingMomentOfInertiaUnits.MillimeterToTheSixth:
            return (value * 1e18)
        
        if from_unit == WarpingMomentOfInertiaUnits.FootToTheSixth:
            return (value / math.pow(0.3048, 6))
        
        if from_unit == WarpingMomentOfInertiaUnits.InchToTheSixth:
            return (value / math.pow(2.54e-2, 6))
        
        return None


    def __convert_to_base(self, value: float, to_unit: WarpingMomentOfInertiaUnits) -> float:
        
        if to_unit == WarpingMomentOfInertiaUnits.MeterToTheSixth:
            return (value)
        
        if to_unit == WarpingMomentOfInertiaUnits.DecimeterToTheSixth:
            return (value / 1e6)
        
        if to_unit == WarpingMomentOfInertiaUnits.CentimeterToTheSixth:
            return (value / 1e12)
        
        if to_unit == WarpingMomentOfInertiaUnits.MillimeterToTheSixth:
            return (value / 1e18)
        
        if to_unit == WarpingMomentOfInertiaUnits.FootToTheSixth:
            return (value * math.pow(0.3048, 6))
        
        if to_unit == WarpingMomentOfInertiaUnits.InchToTheSixth:
            return (value * math.pow(2.54e-2, 6))
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_meters_to_the_sixth(meters_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in meters_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in meters_to_the_sixth.
        :type meters_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(meters_to_the_sixth, WarpingMomentOfInertiaUnits.MeterToTheSixth)

    
    @staticmethod
    def from_decimeters_to_the_sixth(decimeters_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in decimeters_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in decimeters_to_the_sixth.
        :type decimeters_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(decimeters_to_the_sixth, WarpingMomentOfInertiaUnits.DecimeterToTheSixth)

    
    @staticmethod
    def from_centimeters_to_the_sixth(centimeters_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in centimeters_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in centimeters_to_the_sixth.
        :type centimeters_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(centimeters_to_the_sixth, WarpingMomentOfInertiaUnits.CentimeterToTheSixth)

    
    @staticmethod
    def from_millimeters_to_the_sixth(millimeters_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in millimeters_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in millimeters_to_the_sixth.
        :type millimeters_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(millimeters_to_the_sixth, WarpingMomentOfInertiaUnits.MillimeterToTheSixth)

    
    @staticmethod
    def from_feet_to_the_sixth(feet_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in feet_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in feet_to_the_sixth.
        :type feet_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(feet_to_the_sixth, WarpingMomentOfInertiaUnits.FootToTheSixth)

    
    @staticmethod
    def from_inches_to_the_sixth(inches_to_the_sixth: float):
        """
        Create a new instance of WarpingMomentOfInertia from a value in inches_to_the_sixth.

        

        :param meters: The WarpingMomentOfInertia value in inches_to_the_sixth.
        :type inches_to_the_sixth: float
        :return: A new instance of WarpingMomentOfInertia.
        :rtype: WarpingMomentOfInertia
        """
        return WarpingMomentOfInertia(inches_to_the_sixth, WarpingMomentOfInertiaUnits.InchToTheSixth)

    
    @property
    def meters_to_the_sixth(self) -> float:
        """
        
        """
        if self.__meters_to_the_sixth != None:
            return self.__meters_to_the_sixth
        self.__meters_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.MeterToTheSixth)
        return self.__meters_to_the_sixth

    
    @property
    def decimeters_to_the_sixth(self) -> float:
        """
        
        """
        if self.__decimeters_to_the_sixth != None:
            return self.__decimeters_to_the_sixth
        self.__decimeters_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.DecimeterToTheSixth)
        return self.__decimeters_to_the_sixth

    
    @property
    def centimeters_to_the_sixth(self) -> float:
        """
        
        """
        if self.__centimeters_to_the_sixth != None:
            return self.__centimeters_to_the_sixth
        self.__centimeters_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.CentimeterToTheSixth)
        return self.__centimeters_to_the_sixth

    
    @property
    def millimeters_to_the_sixth(self) -> float:
        """
        
        """
        if self.__millimeters_to_the_sixth != None:
            return self.__millimeters_to_the_sixth
        self.__millimeters_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.MillimeterToTheSixth)
        return self.__millimeters_to_the_sixth

    
    @property
    def feet_to_the_sixth(self) -> float:
        """
        
        """
        if self.__feet_to_the_sixth != None:
            return self.__feet_to_the_sixth
        self.__feet_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.FootToTheSixth)
        return self.__feet_to_the_sixth

    
    @property
    def inches_to_the_sixth(self) -> float:
        """
        
        """
        if self.__inches_to_the_sixth != None:
            return self.__inches_to_the_sixth
        self.__inches_to_the_sixth = self.__convert_from_base(WarpingMomentOfInertiaUnits.InchToTheSixth)
        return self.__inches_to_the_sixth

    
    def to_string(self, unit: WarpingMomentOfInertiaUnits = WarpingMomentOfInertiaUnits.MeterToTheSixth, fractional_digits: int = None) -> str:
        """
        Format the WarpingMomentOfInertia to a string.
        
        Note: the default format for WarpingMomentOfInertia is MeterToTheSixth.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the WarpingMomentOfInertia. Default is 'MeterToTheSixth'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == WarpingMomentOfInertiaUnits.MeterToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.meters_to_the_sixth, fractional_digits)} m⁶"""
        
        if unit == WarpingMomentOfInertiaUnits.DecimeterToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.decimeters_to_the_sixth, fractional_digits)} dm⁶"""
        
        if unit == WarpingMomentOfInertiaUnits.CentimeterToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.centimeters_to_the_sixth, fractional_digits)} cm⁶"""
        
        if unit == WarpingMomentOfInertiaUnits.MillimeterToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.millimeters_to_the_sixth, fractional_digits)} mm⁶"""
        
        if unit == WarpingMomentOfInertiaUnits.FootToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.feet_to_the_sixth, fractional_digits)} ft⁶"""
        
        if unit == WarpingMomentOfInertiaUnits.InchToTheSixth:
            return f"""{super()._truncate_fraction_digits(self.inches_to_the_sixth, fractional_digits)} in⁶"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: WarpingMomentOfInertiaUnits = WarpingMomentOfInertiaUnits.MeterToTheSixth) -> str:
        """
        Get WarpingMomentOfInertia unit abbreviation.
        Note! the default abbreviation for WarpingMomentOfInertia is MeterToTheSixth.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.MeterToTheSixth:
            return """m⁶"""
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.DecimeterToTheSixth:
            return """dm⁶"""
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.CentimeterToTheSixth:
            return """cm⁶"""
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.MillimeterToTheSixth:
            return """mm⁶"""
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.FootToTheSixth:
            return """ft⁶"""
        
        if unit_abbreviation == WarpingMomentOfInertiaUnits.InchToTheSixth:
            return """in⁶"""
        


from __future__ import annotations

from functools import total_ordering


@total_ordering
class AbstractMeasure:
    _value: float

    def __str__(self):
        return self.to_string()

    def __add__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value + other._value)

    def __mul__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value * other._value)

    def __sub__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value - other._value)

    def __truediv__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value / other._value)

    def __mod__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value % other._value)

    def __pow__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value ** other._value)

    def __eq__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return self._value == other._value

    def __lt__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return self._value < other._value
    
    def _truncate_fraction_digits(self, num: float, fractional_digits: int):
        if fractional_digits is None:
            return num
        # Convert the number to a string with the desired precision
        num_string = format(num, f".{fractional_digits}f")
        
        # Parse the string back to a number
        truncated_num = float(num_string)
        
        if truncated_num % 1 == 0:  # Check if num is an integer, if so convert it to int, so the X.0 will be printed as X only
            return int(truncated_num)

        return truncated_num



from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricSurfaceChargeDensityUnits(Enum):
        """
            ElectricSurfaceChargeDensityUnits enumeration
        """
        
        CoulombPerSquareMeter = 'CoulombPerSquareMeter'
        """
            
        """
        
        CoulombPerSquareCentimeter = 'CoulombPerSquareCentimeter'
        """
            
        """
        
        CoulombPerSquareInch = 'CoulombPerSquareInch'
        """
            
        """
        

class ElectricSurfaceChargeDensityDto:
    """
    A DTO representation of a ElectricSurfaceChargeDensity

    Attributes:
        value (float): The value of the ElectricSurfaceChargeDensity.
        unit (ElectricSurfaceChargeDensityUnits): The specific unit that the ElectricSurfaceChargeDensity value is representing.
    """

    def __init__(self, value: float, unit: ElectricSurfaceChargeDensityUnits):
        """
        Create a new DTO representation of a ElectricSurfaceChargeDensity

        Parameters:
            value (float): The value of the ElectricSurfaceChargeDensity.
            unit (ElectricSurfaceChargeDensityUnits): The specific unit that the ElectricSurfaceChargeDensity value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricSurfaceChargeDensity
        """
        self.unit: ElectricSurfaceChargeDensityUnits = unit
        """
        The specific unit that the ElectricSurfaceChargeDensity value is representing
        """

    def to_json(self):
        """
        Get a ElectricSurfaceChargeDensity DTO JSON object representing the current unit.

        :return: JSON object represents ElectricSurfaceChargeDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricSurfaceChargeDensity DTO from a json representation.

        :param data: The ElectricSurfaceChargeDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerSquareMeter"}
        :return: A new instance of ElectricSurfaceChargeDensityDto.
        :rtype: ElectricSurfaceChargeDensityDto
        """
        return ElectricSurfaceChargeDensityDto(value=data["value"], unit=ElectricSurfaceChargeDensityUnits(data["unit"]))


class ElectricSurfaceChargeDensity(AbstractMeasure):
    """
    In electromagnetism, surface charge density is a measure of the amount of electric charge per surface area.

    Args:
        value (float): The value.
        from_unit (ElectricSurfaceChargeDensityUnits): The ElectricSurfaceChargeDensity unit to create from, The default unit is CoulombPerSquareMeter
    """
    def __init__(self, value: float, from_unit: ElectricSurfaceChargeDensityUnits = ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__coulombs_per_square_meter = None
        
        self.__coulombs_per_square_centimeter = None
        
        self.__coulombs_per_square_inch = None
        

    def convert(self, unit: ElectricSurfaceChargeDensityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricSurfaceChargeDensityUnits = ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter) -> ElectricSurfaceChargeDensityDto:
        """
        Get a new instance of ElectricSurfaceChargeDensity DTO representing the current unit.

        :param hold_in_unit: The specific ElectricSurfaceChargeDensity unit to store the ElectricSurfaceChargeDensity value in the DTO representation.
        :type hold_in_unit: ElectricSurfaceChargeDensityUnits
        :return: A new instance of ElectricSurfaceChargeDensityDto.
        :rtype: ElectricSurfaceChargeDensityDto
        """
        return ElectricSurfaceChargeDensityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricSurfaceChargeDensityUnits = ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter):
        """
        Get a ElectricSurfaceChargeDensity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricSurfaceChargeDensity unit to store the ElectricSurfaceChargeDensity value in the DTO representation.
        :type hold_in_unit: ElectricSurfaceChargeDensityUnits
        :return: JSON object represents ElectricSurfaceChargeDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_surface_charge_density_dto: ElectricSurfaceChargeDensityDto):
        """
        Obtain a new instance of ElectricSurfaceChargeDensity from a DTO unit object.

        :param electric_surface_charge_density_dto: The ElectricSurfaceChargeDensity DTO representation.
        :type electric_surface_charge_density_dto: ElectricSurfaceChargeDensityDto
        :return: A new instance of ElectricSurfaceChargeDensity.
        :rtype: ElectricSurfaceChargeDensity
        """
        return ElectricSurfaceChargeDensity(electric_surface_charge_density_dto.value, electric_surface_charge_density_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricSurfaceChargeDensity from a DTO unit json representation.

        :param data: The ElectricSurfaceChargeDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerSquareMeter"}
        :return: A new instance of ElectricSurfaceChargeDensity.
        :rtype: ElectricSurfaceChargeDensity
        """
        return ElectricSurfaceChargeDensity.from_dto(ElectricSurfaceChargeDensityDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricSurfaceChargeDensityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter:
            return (value)
        
        if from_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter:
            return (value / 1.0e4)
        
        if from_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch:
            return (value / 1.5500031000062000e3)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricSurfaceChargeDensityUnits) -> float:
        
        if to_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter:
            return (value)
        
        if to_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter:
            return (value * 1.0e4)
        
        if to_unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch:
            return (value * 1.5500031000062000e3)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_coulombs_per_square_meter(coulombs_per_square_meter: float):
        """
        Create a new instance of ElectricSurfaceChargeDensity from a value in coulombs_per_square_meter.

        

        :param meters: The ElectricSurfaceChargeDensity value in coulombs_per_square_meter.
        :type coulombs_per_square_meter: float
        :return: A new instance of ElectricSurfaceChargeDensity.
        :rtype: ElectricSurfaceChargeDensity
        """
        return ElectricSurfaceChargeDensity(coulombs_per_square_meter, ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter)

    
    @staticmethod
    def from_coulombs_per_square_centimeter(coulombs_per_square_centimeter: float):
        """
        Create a new instance of ElectricSurfaceChargeDensity from a value in coulombs_per_square_centimeter.

        

        :param meters: The ElectricSurfaceChargeDensity value in coulombs_per_square_centimeter.
        :type coulombs_per_square_centimeter: float
        :return: A new instance of ElectricSurfaceChargeDensity.
        :rtype: ElectricSurfaceChargeDensity
        """
        return ElectricSurfaceChargeDensity(coulombs_per_square_centimeter, ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter)

    
    @staticmethod
    def from_coulombs_per_square_inch(coulombs_per_square_inch: float):
        """
        Create a new instance of ElectricSurfaceChargeDensity from a value in coulombs_per_square_inch.

        

        :param meters: The ElectricSurfaceChargeDensity value in coulombs_per_square_inch.
        :type coulombs_per_square_inch: float
        :return: A new instance of ElectricSurfaceChargeDensity.
        :rtype: ElectricSurfaceChargeDensity
        """
        return ElectricSurfaceChargeDensity(coulombs_per_square_inch, ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch)

    
    @property
    def coulombs_per_square_meter(self) -> float:
        """
        
        """
        if self.__coulombs_per_square_meter != None:
            return self.__coulombs_per_square_meter
        self.__coulombs_per_square_meter = self.__convert_from_base(ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter)
        return self.__coulombs_per_square_meter

    
    @property
    def coulombs_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__coulombs_per_square_centimeter != None:
            return self.__coulombs_per_square_centimeter
        self.__coulombs_per_square_centimeter = self.__convert_from_base(ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter)
        return self.__coulombs_per_square_centimeter

    
    @property
    def coulombs_per_square_inch(self) -> float:
        """
        
        """
        if self.__coulombs_per_square_inch != None:
            return self.__coulombs_per_square_inch
        self.__coulombs_per_square_inch = self.__convert_from_base(ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch)
        return self.__coulombs_per_square_inch

    
    def to_string(self, unit: ElectricSurfaceChargeDensityUnits = ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the ElectricSurfaceChargeDensity to a string.
        
        Note: the default format for ElectricSurfaceChargeDensity is CoulombPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricSurfaceChargeDensity. Default is 'CoulombPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.coulombs_per_square_meter, fractional_digits)} C/m²"""
        
        if unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.coulombs_per_square_centimeter, fractional_digits)} C/cm²"""
        
        if unit == ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.coulombs_per_square_inch, fractional_digits)} C/in²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricSurfaceChargeDensityUnits = ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter) -> str:
        """
        Get ElectricSurfaceChargeDensity unit abbreviation.
        Note! the default abbreviation for ElectricSurfaceChargeDensity is CoulombPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricSurfaceChargeDensityUnits.CoulombPerSquareMeter:
            return """C/m²"""
        
        if unit_abbreviation == ElectricSurfaceChargeDensityUnits.CoulombPerSquareCentimeter:
            return """C/cm²"""
        
        if unit_abbreviation == ElectricSurfaceChargeDensityUnits.CoulombPerSquareInch:
            return """C/in²"""
        